/***************************************************************
Blob Man v2.0                                      23 April 1999
=============
by Peter Houston <houston.graphics@iname.com>
                < http://members.xoom.com/HoustonGraph/ >
based on Blob Man model by Govert Zoethout <zoethout@gmx.net>
                < http://victorian.fortunecity.com/dada/507/ >

Refer to blob_man.txt for full usage details.

****************************************************************/

#local Blob_Man_Version = version;
#version 3.1;

// Define Default Skin and Lip Textures
#ifndef (Skin_Tex)   
        #declare Skin_Tex=
                texture {
                         pigment{rgb <.9,.7,.6>}
                         finish{phong .3 phong_size 40}
                         normal{crackle .2 turbulence .7 scale .7}
                }                         
#end

#ifndef (Lip_Tex)   
        #declare Lip_Tex =
                texture {
                         pigment{rgb <.7,.3,.2>}
                         finish{phong .3 phong_size 40}
                         normal{crackle .2 turbulence .7 scale .7}
                }                         
#end

/***************************************************************
Blob_Man Macro
- - - - - - - - 
Usage : Declare Pose Arrays before calling this macro, 
        example can be found in blob_man.pos file.
        
Blob_Man( PoseNo , Build , Face_Map , Eye_Colour , Right_Lid , Left_Lid , 
          HairStyle , Hair_Col , Right_Hand , Left_Hand )        

PoseNo     = Array Entry for pose required
Build      = Blob Threshold Level    - standard: 0.5
Face_Map   = Image Map used for face 0= No Map 1-5 is number of png file.
             Name of file is bmface#.png where # is Face_Map Number, uses aplha channel.
Eye_Colour = Eye Iris Colour <r,g,b>
Right_Lid  = Right Eye Lid Position  (range : 0-Open to 1-Closed)      - standard: 0
Left_Lid   = Left Eye Lid Position  (range : 0-Open to 1-Closed)       - standard: 0
HairStyle  = Array Entry for Hair Style required
Hair_Col   = Hair Colour (<r,g,b>)
Left_Hand  = Array Entry for Hand Position to overide standard position as defined 
             by the Pose Definition, 0=Pose Definition
Right_Hand = Array Entry for Hand Position to overide standard position as defined 
             by the Pose Definition, 0=Pose Definition                                                        
 
****************************************************************/
#macro Blob_Man(PoseNo,Build,Face_Map,Eye_Colour,Right_Lid,Left_Lid,HairStyle,Hair_Col,Right_Hand,Left_Hand)
        // Use Hands defined by PoseNo if not specified in Macro Call.
        #if (Left_Hand=0)  
                #local Left_Hand  = Pose_2[PoseNo][5]; 
        #end
        #if (Right_Hand=0) 
                #local Right_Hand = Pose_2[PoseNo][6]; 
        #end
        
        #local RA_S2E     = Pose_1[PoseNo][0];  
        #local RA_E2W     = Pose_1[PoseNo][1];  
        #local LA_S2E     = Pose_1[PoseNo][2];  
        #local LA_E2W     = Pose_1[PoseNo][3];    
        #local RL_H2K     = Pose_1[PoseNo][4];  
        #local RL_K2A     = Pose_1[PoseNo][5];  
        #local LL_H2K     = Pose_1[PoseNo][6]; 
        #local LL_K2A     = Pose_1[PoseNo][7];    
        #local RH_Rot     = Pose_1[PoseNo][8];  
        #local LH_Rot     = Pose_1[PoseNo][9];
        #local RF_Rot     = Pose_1[PoseNo][10];
        #local LF_Rot     = Pose_1[PoseNo][11]; 
               
        #local Eye_Rot    = Pose_2[PoseNo][0];  
        #local Lip_Size   = Pose_2[PoseNo][1];         
        #local Head_Twist = Pose_2[PoseNo][2];
        #local Head_Tip   = Pose_2[PoseNo][3];
        #local Head_Tilt  = Pose_2[PoseNo][4];

        #local Hair_Rot   = Pose_Hair_1[HairStyle][0];

        #local Hair_On    = Pose_Hair_2[HairStyle][0];
        #local Hair_Arch  = Pose_Hair_2[HairStyle][1];
        #local Hair_Thck  = Pose_Hair_2[HairStyle][2];
        #local Hair_Line  = Pose_Hair_2[HairStyle][3];
        #local Hair_LDpth = Pose_Hair_2[HairStyle][4];
        #local Hair_Wght  = Pose_Hair_2[HairStyle][5];
        #local Hair_Hght  = Pose_Hair_2[HairStyle][6];
        #local Hair_Length= Pose_Hair_2[HairStyle][7];
        #local Hair_Curl  = Pose_Hair_2[HairStyle][8];

        #local LF_Spread  = Pose_Hands[Left_Hand][0];
        #local LFP_Pos    = Pose_Hands[Left_Hand][1];
        #local LFR_Pos    = Pose_Hands[Left_Hand][2];
        #local LFM_Pos    = Pose_Hands[Left_Hand][3];
        #local LFI_Pos    = Pose_Hands[Left_Hand][4];
        #local LFT_Pos    = Pose_Hands[Left_Hand][5];
        #local RF_Spread  = Pose_Hands[Right_Hand][0];
        #local RFP_Pos    = Pose_Hands[Right_Hand][1];
        #local RFR_Pos    = Pose_Hands[Right_Hand][2];
        #local RFM_Pos    = Pose_Hands[Right_Hand][3];
        #local RFI_Pos    = Pose_Hands[Right_Hand][4];
        #local RFT_Pos    = Pose_Hands[Right_Hand][5];
        
        #declare BlobMan= 
                merge {
                        object { head( Build,Eye_Colour,Right_Lid,Left_Lid,Eye_Rot,Lip_Size,Head_Twist,Head_Tip,Head_Tilt,Face_Map ) }
                        object {torso( Build,RA_S2E,RA_E2W,LA_S2E,LA_E2W,RL_H2K,RL_K2A,LL_H2K,LL_K2A ) }
                        object { hand( Build,LA_S2E,LA_E2W,LH_Rot, 1,LF_Spread, LFP_Pos, LFR_Pos, LFM_Pos, LFI_Pos, LFT_Pos ) }
                        object { hand( Build,RA_S2E,RA_E2W,RH_Rot,-1,RF_Spread, RFP_Pos, RFR_Pos, RFM_Pos, RFI_Pos, RFT_Pos ) }
                        object { feet( Build,RL_H2K,RL_K2A,LL_H2K,LL_K2A,RF_Rot,LF_Rot ) }
                        object { hair( Hair_On,Hair_Rot,Hair_Col,Hair_Arch,Hair_Thck,Hair_Line,Hair_LDpth,Hair_Wght,Hair_Hght,Hair_Length,Hair_Curl,Head_Twist,Head_Tip,Head_Tilt ) }
                } 
#end

/***************************************************************
Torso Macro
- - - - - - 
Usage :

torso(build , skin , RA_S2E , RA_E2W , LA_S2E , LA_E2W , 
                     RL_H2K , RL_K2A , LL_H2K , LL_K2A  )

build  = Blob Threshold Level                        - standard: 0.5
skin   = Skin color (<r,g,b>)                        - standard: <.9,.7,.6>
RA_S2E = Right Arm Shoulder to Elbow rotation vector - standard: <0,0,0>
RA_E2W = Right Arm Elbow to Wrist rotation vector    - standard: <0,0,0>
LA_S2E = Left Arm Shoulder to Elbow rotation vector  - standard: <0,0,0>
LA_E2W = Left Arm Elbow to Wrist rotation vector     - standard: <0,0,0>
RL_H2K = Right Leg Hip to Knee rotation vector       - standard: <0,0,0>
RL_K2A = Right Leg Knee to Ankle rotation vector     - standard: <0,0,0>
LL_H2K = Right Leg Hip to Knee rotation vector       - standard: <0,0,0>
LL_K2A = Right Leg Knee to Ankle rotation vector     - standard: <0,0,0>

*****************************************************************/

#macro torso(build,RA_S2E,RA_E2W,LA_S2E,LA_E2W,RL_H2K,RL_K2A,LL_H2K,LL_K2A)
    
        #local UpperarmLength=11; 
        #local ForearmLength =9;  
        #local ThighLength   =14; 
        #local CalfLength    =17; 

        #local R_Elbow=vrotate(<0,-UpperarmLength,0>,RA_S2E);
        #local R_Wrist=vrotate(<0,-ForearmLength,0> ,RA_E2W);
        #local L_Elbow=vrotate(<0,-UpperarmLength,0>,LA_S2E);
        #local L_Wrist=vrotate(<0,-ForearmLength,0> ,LA_E2W);

        #local R_Knee =vrotate(<0,-ThighLength,0>,RL_H2K);
        #local R_Ankle=vrotate(<0,-CalfLength,0> ,RL_K2A);
        #local L_Knee =vrotate(<0,-ThighLength,0>,LL_H2K);
        #local L_Ankle=vrotate(<0,-CalfLength,0> ,LL_K2A);

        #local R_Upperarm=transform {rotate RA_S2E translate <-9,-UpperarmLength,1>}
        #local L_Upperarm=transform {rotate LA_S2E translate < 9,-UpperarmLength,1>}
        #local R_Forearm =transform {rotate RA_E2W translate R_Elbow translate <-9,-UpperarmLength,1>}
        #local L_Forearm =transform {rotate LA_E2W translate L_Elbow translate < 9,-UpperarmLength,1>}
    
        #local R_Thigh=transform {rotate RL_H2K translate <-4,-29,0>}
        #local L_Thigh=transform {rotate LL_H2K translate < 4,-29,0>}
        #local R_Calf =transform {rotate RL_K2A translate R_Knee translate <-4,-29,0>} 
        #local L_Calf =transform {rotate LL_K2A translate L_Knee translate < 4,-29,0>}
    
        blob{
                threshold build-.3          
// Neck & Spine
                #local Loop=0;
                #while (Loop>-29)
                        sphere{0,2,2 translate<0,Loop-3,sin((Loop+2)*pi/(4+abs(Loop)/4))*(.25+abs(Loop/15))+1>}             
                        #if (mod(Loop+2,3)=0)
                                sphere{0,1,2 translate<0,Loop-3,sin((Loop+2)*pi/(4+abs(Loop)/4))*(.25+abs(Loop/15))+2.1>}             
                        #end
                        #local Loop=Loop-1;
                #end

//Shoulders & Neck
                cylinder{<-8,-9,1><-1,-6.75,1>,1.2,2}
                cylinder{< 8,-9,1><1,-6.75,1>,1.2,2}
                cylinder{<-8,-10,1><8,-10,1>,3.2,2}
//Arms
// Upperarms
// Right
                cylinder{<0,0,0>,<0,-UpperarmLength,0>,2.7,2 transform R_Upperarm}
                sphere  {<0,0,0>3.5,2.5 scale <1,UpperarmLength*.2,1> translate <0,-UpperarmLength*.2-1,0> transform R_Upperarm}
// Left
                cylinder{<0,0,0>,<0,-UpperarmLength,0>,2.7,2 transform L_Upperarm}
                sphere  {<0,0,0>3.5,2.5 scale <1,UpperarmLength*.2,1> translate <0,-UpperarmLength*.2-1,0> transform L_Upperarm}
// Forearms
// Right
                cylinder{0,<0,-ForearmLength,0>,2.5,2 transform R_Forearm }
                sphere  {0,3,2 scale <1,ForearmLength*.18,1> translate <0,-ForearmLength*.18-.5,0> transform R_Forearm}
                sphere  {0,2.75,-1.5 scale <1,3.5,1> translate <0,-ForearmLength-1,0> transform R_Forearm}
// Left
                cylinder{0,<0,-ForearmLength,0>,2.5,2  transform L_Forearm }
                sphere  {0,3,2 scale <1,ForearmLength*.18,1> translate <0,-ForearmLength*.18-.5,0> transform L_Forearm}
                sphere  {0,2.75,-1.5 scale <1,3.5,1> translate <0,-ForearmLength-1,0> transform L_Forearm}
// Upper Chest
                cylinder{<-8,-9,.5><-1,-9.5,-1.25>,1,2 }
                cylinder{<8,-9,.5><1,-9.5,-1.25>,1,2 }
                cylinder{<-6,-10.5,-.5><6,-10.5,-.5>,2,2}
// Chest and Shoulder Blades    
                sphere{0,4,2 scale<1.4,2,1.3> rotate x*20 translate<-2.8,-14,0> } // Right
                sphere{0,4,2 scale<1.4,2,1.3> rotate x*20 translate< 2.8,-14,0>}  // Left
// Side
                sphere{0,3,2 scale<1.5,2,1.2> translate<-2.8,-22.5,0> } // Right
                sphere{0,3,2 scale<1.5,2,1.2> translate<2.8,-22.5,0>}   // Left
// Abdomen
                sphere{0,3,2 scale<1,1,1.2>   translate<-1.5,-20,-1>}
                sphere{0,3,2 scale<1,1,1.2>   translate<1.5,-20,-1>}
                sphere{0,3,2 scale<1,1.2,1.2> translate<-1.5,-22.5,-1.25>}
                sphere{0,3,2 scale<1,1.2,1.2> translate<1.5,-22.5,-1.25>}
                sphere{0,3,2 scale<1,1,1.2>   translate<-1.5,-25,-1>}
                sphere{0,3,2 scale<1,1,1.2>   translate<1.5,-25,-1>}
// Pelvis
                sphere{0,5,2 scale<1.2,1.6,1>  translate<0,-27,-.5> } //Front & Back
                sphere{0,3,2 scale<1.4,.8,1.5> rotate z*-75 translate<-4,-28,-.5>} // Right
                sphere{0,3,2 scale<1.4,.8,1.5> rotate z* 75 translate< 4,-28,-.5>}  // Left 
// Buttocks      
                sphere{0,3,2 scale<1.75,2,1.5> rotate x*15 translate<-3.,-29.5,1.5> } // Right
                sphere{0,3,2 scale<1.75,2,1.5> rotate x*15 translate< 3.,-29.5,1.5>} // Left
// Legs
// Thighs
// Right
                cylinder{<0,0,0>,<0,-ThighLength,0>,4,4  transform R_Thigh}
                sphere  {<0,0,0>5,4 scale <1,ThighLength*.15,1> translate <0,-ThighLength*.15-2,0> transform R_Thigh}
                sphere  {<0,0,0>,4,-2.5 scale <1,2,1> translate y*-ThighLength transform R_Thigh} 
                sphere  {<0,0,0>,4, 2.5 translate y*-(ThighLength+1.5) transform R_Thigh} 
// Left
                cylinder{<0,0,0>,<0,-ThighLength,0>,4,4  transform L_Thigh}
                sphere  {<0,0,0>5,4 scale <1,ThighLength*.15,1> translate <0,-ThighLength*.15-2,0> transform L_Thigh}
                sphere  {<0,0,0>,4,-2.5 scale <1,2,1> translate y*-ThighLength transform L_Thigh} 
                sphere  {<0,0,0>,4, 2.5 translate y*-(ThighLength+1.5) transform L_Thigh}      
// Calves     
// Right
                cylinder{<0,0,0>,<0,-CalfLength,0>,3,2 transform R_Calf}
                sphere  {<0,0,0>4,4 scale <1,CalfLength*.15,1> translate <0,-CalfLength*.15-3,0> transform R_Calf}
                sphere  {<0,0,0>,3,-1.25 scale <1,2,1> translate y*-CalfLength transform R_Calf} 
// Left      
                cylinder{<0,0,0>,<0,-CalfLength,0>,3,2 transform L_Calf}
                sphere  {<0,0,0>4,4 scale <1,CalfLength*.15,1> translate <0,-CalfLength*.15-3,0> transform L_Calf}
                sphere  {<0,0,0>,3,-1.25 scale <1,2,1> translate y*-CalfLength transform L_Calf} 
// Skin Texture
                texture {Skin_Tex}
        }
//Origin Variables
        #declare Origin_RL=< 4,31,0>-R_Ankle-R_Knee;
        #declare Origin_LL=<-4,31,0>-L_Ankle-L_Knee;
        #declare Origin_RA=< 9,20,0>-R_Elbow-R_Wrist;
        #declare Origin_LA=<-9,20,0>-L_Elbow-L_Wrist;
#end

/***************************************************************

Head Macro
- - - - - 
Usage :

head( build , eye , R_Lid, L_Lid , EyeRot , lip , skin , Twist , Tip , Tilt )

build     = Blob Threshold Level                                      - standard: .5
eye       = Iris Colour of eye (<r,g,b>) - i.e. <.75,1,1> = Pale Blue
R_Lid     = Right Eye Lid Position  (range : 0-Open to 1-Closed)      - standard: 0
L_Lid     = Left Eye Lid Position  (range : 0-Open to 1-Closed)       - standard: 0
EyeRot    = Eye Rotation Angle left-right                             - standard: 0
lip       = Mouth Size (range: -0.5 to 0.5 )                          - standard: 0
skin      = Skin color (<r,g,b>)                                      - standard: <.9,.7,.6>
Twist     = Head rotation Right to Left (range: -85 to 85)            - standard: 0
Tip       = Head position Front to Back (range: -15 to 50)            - standard: 0
Tilt      = Head position Right Side to Left Side  (range: -25 to 25) - standard: 0
Face_Map  = Image Map used for face 0= No Map 1-5 is number of png file.
            Name of file is bmface#.png where # is Face_Map Number, uses aplha channel.

*****************************************************************/                                                                                

#macro head(build,eye,R_Lid,L_Lid,EyeRot,lip,Twist,Tip,Tilt,Face_Map)
        union{
                blob {
                        threshold build
                        sphere{0,5.6,2  } // Top of Skull
                        sphere{0,4.2,2 scale<1,1.4,1> translate <0,-1.4,-.7> } //Lower Jaw
                        sphere{0,2.8,2 scale<1,1.4,1> translate <0,-2.8,-1.4>} // Chin
                        sphere{0,1,1  scale <2,1,1> translate <0,-5.,-2.75>} // Chin
// Brow Ridge
                        #local Loop=0;
                        #while (Loop<90)
                                sphere{<0,.8,-3.25>.009*(90-Loop),3.5 rotate y*Loop translate <0,-.009*(90-Loop)+.25,-.65>  }
                                sphere{<0,.8,-3.25>.009*(90-Loop),3.5 rotate y*-Loop translate <0,-.009*(90-Loop)+.25,-.65>  }              
                                #local Loop=Loop+5;
                        #end
// Eye Socket
                        sphere{0,2,-1.75 scale<1.5,.7,1> translate<0,-.65,-4> rotate y*35 }
                        sphere{0,2,-1.75 scale<1.5,.7,1> translate<0,-.65,-4> rotate y*-35 }
// Cheeks     
                        sphere{0,3,-1.75 scale <1,1,1> translate <-2.75,-2.5,-3.5> } 
                        sphere{0,3,-1.75 scale <1,1,1> translate <2.75,-2.5,-3.5> }
// Nose
                        cylinder{<0,1,.5><0,-1,-.5>,1,3 translate <0,-.7,-3.5> } // Nose
                        cylinder{<-.7,0,0><.7,0,0>,.5,2 translate<0,-1.75,-3.85> }  // Nostrils
// Ears
                        sphere{0,1,2 scale<1,1.75,.7> rotate y*45 rotate z*15 translate<-4.2,-.85,0> } // Outer Ear Right
                        sphere{0,1,-2 scale<1,1.75,.7>*.7 rotate y*45 rotate z*15 translate<-4.2,-.7,-.35>} // Inner Ear Right
                        sphere{0,1,2 scale<1,1.75,.7>/3.5 rotate y*45 rotate z*15 translate<-4.025,-.7,-.35> } // Inner Ear Canal Right
                        sphere{0,1,2 scale<1,1.75,.7> rotate y*-45 rotate z*-15 translate<4.2,-.85,0> } // Outer Ear Left
                        sphere{0,1,-2 scale<1,1.75,.7>*.7 rotate y*-45 rotate z*-15 translate<4.2,-.7,-.35>} // Inner Ear Right
                        sphere{0,1,2 scale<1,1.75,.7>/3.5 rotate y*-45 rotate z*-15 translate<4.025,-.7,-.35>} // Inner Ear Canal Right
// Lips 
                        sphere{0,1.4,2 scale<.85+lip,.35,1> rotate x*21 translate<0,-3.5,-3.05> rotate y*7 texture {Lip_Tex}} //  Upper Right
                        sphere{0,1.4,2 scale<.85+lip,.35,1> rotate x*21 translate<0,-3.5,-3.05> rotate y*-7 texture {Lip_Tex} } // Upper Left
                        sphere{0,1.4,-1 scale<1+lip,.21,.5> rotate x*-21 translate<0,-3.675,-3.5> texture {Lip_Tex} } // Lower Right
                        sphere{0,1.4,2 scale<1+lip,.35,.56> rotate x*-42 translate<0,-3.85,-3.15> texture {Lip_Tex} } // Lower Left 
// Eyes Sockets
                        sphere{0,1,-2 scale<1,.7,1> translate<0,-.35/*-lid*/,-3.55> rotate y*40 } //Right Eye Socket
                        sphere{0,1,-2 scale<1,.7,1> translate<0,-.35/*-lid*/,-3.55> rotate y*-40} // Left Eye Socket
                }
// Eyes
                union{
                        sphere{0,1 pigment{rgb 1}}
                        sphere{0,.7 translate z*-.35 rotate y*18 pigment{rgb eye}}
                        sphere{0,.2 translate z*-.9  rotate y*18 pigment{rgb 0}}
                        difference {
                                sphere{0,1.1 }
                                box {<5,5,0><-5,-5,-10> }
                                rotate y*9
                                rotate x*-(L_Lid*65+65)
                        }  
                        rotate x*-7
                        rotate y*-15
                        rotate <0,EyeRot+20,0>
                        translate<0,-.35,-2.5> 
                        rotate y*-40
                }
                union{
                        sphere{0,1 pigment{rgb 1}}
                        sphere{0,.7 translate z*-.35 rotate y*-18 pigment{rgb eye}}
                        sphere{0,.2 translate z*-.9  rotate y*-18 pigment{rgb 0}} 
                        difference {
                                sphere{0,1.1 }
                                box {<2,2,0><-2,-2,-4> }
                                rotate y*-9
                                rotate x*-(R_Lid*65+65)
                        }
                        rotate x*-7
                        rotate y*15
                        rotate <0,EyeRot-20,0>
                        translate<0,-.35,-2.5> 
                        rotate y*40 
                }

                material {
                        texture {Skin_Tex}                         
                        #if (Face_Map>0)
                                texture {pigment { 
                                                image_map{ 
                                                        png concat ("bmface", str(Face_Map,0,0),".png")
                                                        map_type 2 
                                                        interpolate 4
                                                        once
                                                } 
                                                rotate y*-90
                                                translate <0,-.55,0>
                                                scale 12
                                        }
                                }
                        #end
                }
// Head Tilt, Tip & Twist    
                scale 1.1
                translate <0,5,-1>
                rotate z*Tilt
                rotate x*Tip
                rotate y*Twist 
                translate <0,-5,1>
        }
#end

/****************************************************************

Hand Macro
- - - - - 
Usage: Call this macro twice, once for each hand.

hand( build , skin , H_S2E , H_E2W , H_Rot , Hand, F_Spread, FP_Pos, FR_Pos, FM_Pos, FI_Pos, FT_Pos )

build    =  Blob Threshold Level                  - standard: 0.5
skin     =  Skin color (<r,g,b>)                  - standard: <.9,.7,.6>
H_S2E    =* Arm Shoulder to Elbow rotation vector - standard: Same as Torso
H_E2W    =* Arm Elbow to Wrist rotation vector    - standard: Same as Torso
H_Rot    =  Hand rotation Vector                  - standard: <0,0,0>                                
Hand     =  Right Hand = -1 / Left Hand = 1
F_Spread =  Finger Spread    - 0 = Closed   , 1 = Relaxed , 2 = Wide
FP_Pos   =  Pinky Position   - 0 = Straight , 1 = Relaxed , 2 = Curled
FR_Pos   =  Ring Position    - 0 = Straight , 1 = Relaxed , 2 = Curled
FM_Pos   =  Middle Position  - 0 = Straight , 1 = Relaxed , 2 = Curled
FI_Pos   =  Index Position   - 0 = Straight , 1 = Relaxed , 2 = Curled
FT_Pos   =  Thumb Position   - 0 = Straight , 1 = Relaxed , 2 = Curled

* These MUST be the same as the Torso Macro if they are to line up correctly.  

*******************************************************************/

#macro hand(build,H_S2E,H_E2W,H_Rot,Hand,F_Spread,FP_Pos,FR_Pos,FM_Pos,FI_Pos,FT_Pos)

        #local UpperarmLength=11;  
        #local ForearmLength = 9;   

        #local H_Elbow=vrotate(<0,-UpperarmLength,0>,H_S2E);
        #local H_Wrist=vrotate(<0,-ForearmLength,0> ,H_E2W);

//Finger Curl Angles
        #if (FP_Pos=1) #local FPA=8;   #else   #local FPA=20*FP_Pos;  #end
        #if (FR_Pos=1) #local FRA=8;   #else   #local FRA=20*FR_Pos;  #end
        #if (FM_Pos=1) #local FMA=8;   #else   #local FMA=20*FM_Pos;  #end
        #if (FI_Pos=1) #local FIA=8;   #else   #local FIA=20*FI_Pos;  #end
        #if (FT_Pos=1) #local FTA=15;  #else   #local FTA=7.5*FT_Pos; #end
//Finger Rotations
        #local FP_Rot=<FP_Pos*-FPA , 0 , 4*F_Spread*2>;
        #local FR_Rot=<FR_Pos*-FRA , 0 , 4*F_Spread  >;
        #local FM_Rot=<FM_Pos*-FMA , 0 , 0           >;
        #local FI_Rot=<FI_Pos*-FIA , 0 ,-4*F_Spread  >;
        #local FT_Rot=< 0 , 20*FT_Pos , FT_Pos*FTA >;
// Finger Section Lengths
        #local FP_L=2.12;  //Lower Pinky
        #local FP_U=1.54;  //Upper Pinky
        #local FR_L=2.76;  //Lower Ring
        #local FR_U=1.72;  //Upper Ring
        #local FM_L=2.92;  //Lower Middle
        #local FM_U=1.75;  //Upper Middle
        #local FI_L=3.01;  //Lower Index
        #local FI_U=1.28;  //Upper Index
        #local FT_L=1.01;  //Lower Thumb
        #local FT_U=1.50;  //Upper Thumb

// Finger Transformations
        #local FPL_Tran=transform {rotate FP_Rot translate < -1.97 , 3.94 , 1.25 > }
        #local FPU_Tran=transform {rotate FP_Rot+FP_Rot+<-10,0,0> translate vrotate(<0,FP_L,0>,FP_Rot) translate < -1.97 , 3.94 , 1.25 >}
        #local FRL_Tran=transform {rotate FR_Rot translate < -0.65 , 3.94 , 1.75 > }
        #local FRU_Tran=transform {rotate FR_Rot+FR_Rot+<-10,0,0> translate vrotate(<0,FR_L,0>,FR_Rot) translate < -0.65 , 3.94 , 1.75 >}
        #local FML_Tran=transform {rotate FM_Rot translate <  0.69 , 3.94 , 1.75 > }
        #local FMU_Tran=transform {rotate FM_Rot+FM_Rot+<-10,0,0> translate vrotate(<0,FM_L,0>,FM_Rot) translate <  0.69 , 3.94 , 1.75 >}
        #local FIL_Tran=transform {rotate FI_Rot translate <  2.02 , 3.48 , 1.25 > }
        #local FIU_Tran=transform {rotate FI_Rot+FI_Rot+<-10,0,0> translate vrotate(<0,FI_L,0>,FI_Rot) translate <  2.02 , 3.48 , 1.25 >}
        #local FTL_Tran=transform {rotate FT_Rot translate < 2.0 , 1.5 ,-.0 >}                                                    
        #local FTU_Tran=transform {rotate FT_Rot+FT_Rot translate < 2.0+FT_L , 1.5+FT_L ,-0.25 >}
        #if (DeBug=on) union { #end
        blob {
                threshold build
                sphere { 0, 1.65, .75 scale <1.85,1.05,.75> rotate y*-25 translate <-1.15,3.8,1.25> } //palm
                sphere { 0, 1.65, .75 scale <1.85,1.075,.75> rotate y*25 translate <1.15,3.8,1.25> }  //palm
                sphere { 0, 1.70, .75 scale <1.78, 1.3,.75> rotate y*-25 translate <-1,2.6,1> }       //midhand
                sphere { 0, 1.70, .75 scale <1.78, 1.3,.75>  rotate y*25 translate <0.69,2.6,1> }     //midhand
                sphere { 0, 1.7, .85 scale <1.4, 1.25,.75> rotate y*-25 translate <-0.55,1.2,0.5> }   //heel
                sphere { 0, 1.7, .85 scale <1.4, 1.25,.75> rotate y*25 translate <0.54,1.2,0.5>}      //heel
                sphere { 0, 1.7, .85 scale <1.3,1,1> translate <0.35,0.5,0.5>}                        //wrist
                sphere { 0, 1.7, -.75 scale <3,3,1> translate <0,2,2.85>}     // counteract bulge on back of hand

                cylinder { <0,0,0><0,FP_L,0>1,1 transform FPL_Tran }          //lower pinky
                cylinder { <0,0,0><0,FP_U,0>1,1 transform FPU_Tran }          //upper pinky
                sphere { 0, 1, -1 transform FPL_Tran }                        //counteract pinky palm bulge
                sphere { 0, 1, -1 transform FPU_Tran }                        //counteract pinky knuckle bulge
                cylinder { <0,0,0><0,FR_L,0>1,1 transform FRL_Tran }          //lower ring
                cylinder { <0,0,0><0,FR_U,0>1,1 transform FRU_Tran }          //upper ring    
                sphere { 0, 1, -1 transform FRL_Tran }                        //counteract ring palm bulge
                sphere { 0, 1, -1 transform FRU_Tran }                        //counteract ring knuckle bulge
                cylinder { <0,0,0><0,FM_L,0>1,1 transform FML_Tran }          //lower middle
                cylinder { <0,0,0><0,FM_U,0>1,1 transform FMU_Tran }          //upper middle
                sphere { 0, 1, -1 transform FML_Tran }                        //counteract middle palm bulge
                sphere { 0, 1, -1 transform FMU_Tran }                        //counteract middle knuckle bulge
                cylinder { <0,0,0><0,FI_L,0>1,1 transform FIL_Tran }          //lower index
                cylinder { <0,0,0><0,FI_U,0>1,1 transform FIU_Tran }          //upper index
                sphere { 0, 1, -1 transform FIL_Tran }                        //counteract index palm bulge
                sphere { 0, 1, -1 transform FIU_Tran }                        //counteract index knuckle bulge
                cylinder { <0,0,0><FT_L,FT_L,-.25>1.25,1 transform FTL_Tran } //lower thumb
                cylinder { <0,0,0><FT_U,FT_U,-.25>1.25,1 transform FTU_Tran } //upper thumb
                sphere { 0, 1.25, -.5 transform FTL_Tran }                    //counteract thumb palm bulge
                sphere { 0, 1.25, -1 transform FTU_Tran }                     //counteract thumb knuckle bulge

                translate <0,0,-0.5>                
                scale <Hand,-1,1>
                scale .85
                
                rotate y*(90*Hand)
                rotate H_Rot
                rotate H_E2W
    
                translate H_Elbow+H_Wrist+<9*Hand,-UpperarmLength,.5>
    
                texture {Skin_Tex}
        }
        #if (DeBug=on)
                object {LimbLoc1                        
                        rotate y*(90*Hand)
                        rotate H_Rot
                        rotate H_E2W
                        translate H_Elbow+H_Wrist+<9*Hand,-UpperarmLength,.5>       
                }
        } 
        #end        
#end

/****************************************************************

Hair Macro
- - - - -
Usage:

hair( Hair_On, Hair_Rot, Hair_Col, Hair_Arch, Hair_Thck, Hair_Line, Hair_LDpth, Hair_Wght, Hair_Hght, Hair_Length, 
      Hair_Curl, Twist, Tip, Tilt )

Hair_On        = Hair (1) or No Hair (0)
Hair_Rot       = Rotation Vector for Hair (<x,y,z>)            - standard: <0,0,0>
Hair_Col       = Hair Colour (<r,g,b>)
Hair_Arch      = Arch hairline over head inline with ears (1) or Not (0)
Hair_Thck      = Thickness of Hair (Best settings 0.15 to 0.5) - standard: 0.25
Hair_Line      = Height of Hairline (Best settings 3 to 6)     - standard: 5
Hair_LDpth     = Hairline profile Depth (Range: 0 to 1)        - standard: 0.7
Hair_Wght      = Hair Weight (Range 0.005 to 0.05)             - standard: 0.01
Hair_Hght      = Hair Height (Range 1 to 2)                    - Standard: 1
Hair_Length    = Hair Length (Range 1 to 20)                   - standard: 10
Hair_Curl      = Curliness (Range 0-Straight to 0.1-Curly)     - Standard: 0.05
Twist          =* Head Twist Angle                             - Standard: Same as Head
Tip            =* Head Tip Angle                               - Standard: Same as Head  
Tilt           =* Head Tilt Angle                              - Standard: Same as Head  

* These MUST be the same as the Head Macro if it is to line up correctly.  

*******************************************************************/

#macro hair(Hair_On,Hair_Rot,Hair_Col,Hair_Arch,Hair_Thck,Hair_Line,Hair_LDpth,Hair_Wght,Hair_Hght,Hair_Length,Hair_Curl,Twist,Tip,Tilt)
        #if (Hair_On=1)       
                union{
                        #local rr=seed(77);
                        #local tt=0;
                        #while (tt<20)
                                #local zz=tt*3+3;   
                                #local cc=0;
                                #while (cc<zz)
                                        #local yy=cc*360/zz+rand(rr)*6;
                                        #local xx=Hair_Line-sin((yy+15)*pi/30)*Hair_LDpth;
                                        #local hh=vrotate(<0,.8,0>,<tt*xx+rand(rr)*5,yy,45>);
                                        #local hx=hh.x;
                                        #local hy=hh.y;
                                        #local hz=hh.z;
                                        #local dx=hx/50;
                                        #local dy=hy/50*Hair_Hght;
                                        #local dz=hz/50;    
                                        #if (Hair_Arch=1 & dx>0)
                                                #local dy=dy+dx;
                                                #local dx=-dx;
                                        #end
                                        #if (hy>-.25)
                                                blob{threshold .5
                                                #local ll=0;           
                                                #local length=Hair_Length-rand(rr)*Hair_Length/5;
                                                #local vv=rand(rr)*length;
                                                #while (ll<length) 
                                                        sphere{<hx+Hair_Curl*sin((vv+ll)*pi/3),hy,hz+Hair_Curl*cos((vv+ll)*pi/3)>,Hair_Thck/5,1}
                                                        #local ll=ll+Hair_Thck;
                                                        #local hx=hx+dx;
                                                        #local hy=hy+dy;
                                                        #local hz=hz+dz;
                                                        #local dx=dx*(1-Hair_Wght*2);
                                                        #local dy=dy*(1-Hair_Wght/2)-Hair_Wght/20;
                                                        #local dz=dz*(1-Hair_Wght);
                                                        #if (Hair_Arch=1 & dx>0)
                                                                #local dy=dy+0.001;       
                                                                #if (hx>-.5)
                                                                        #local hx=hx+dx;
                                                                #end
                                                        #end
                                                #end
                                                #ifndef (Hair_Tex)
                                                        pigment{rgb Hair_Col+<rand(rr)/10,rand(rr)/10,rand(rr)/10>}}
                                                #else
                                                        texture {Hair_Tex}
                                                #end
                                        #end
                                        #local cc=cc+1;
                                #end
                                #local tt=tt+1;
                        #end
                        rotate y*90
                        rotate Hair_Rot
                        scale 4.5
                        translate <0,5,-1>
                        rotate z*Tilt
                        rotate x*Tip
                        rotate y*Twist 
                        translate <0,-5,1>
                }
        #else
                sphere {<0,0,0>.000000001 texture {Skin_Tex}}
        #end
#end


/****************************************************************

Feet Macro
- - - - -
Usage:

feet( build , skin , RF_H2K , RF_K2A , LF_H2K , LF_K2A , RF_Rot , LF_Rot)

build  =  Blob Threshold Level                        - standard: 0.5
skin   =  Skin color (<r,g,b>)                        - standard: <.9,.7,.6>
RF_H2K =* Right Leg Hip to Knee rotation vector       - standard: As Torso
RF_K2A =* Right Leg Knee to Ankle rotation vector     - standard: As Torso
LF_H2K =* Right Leg Hip to Knee rotation vector       - standard: As Torso
LF_K2A =* Right Leg Knee to Ankle rotation vector     - standard: As Torso
RF_Rot =  Right Foot rotation Vector                  - standard: <0,0,0>
LF_Rot =  Left Foot rotation Vector                   - standard: <0,0,0>

* These MUST be the same as the Torso Macro if they are to line up correctly.  

******************************************************************/

#macro feet(build,RF_H2K,RF_K2A,LF_H2K,LF_K2A,RF_Rot,LF_Rot)

        #local ThighLength=14; 
        #local CalfLength =17; 
        
        #local RF_Knee =vrotate(<0,-ThighLength,0>,RF_H2K);
        #local RF_Ankle=vrotate(<0,-CalfLength,0> ,RF_K2A);
        #local LF_Knee =vrotate(<0,-ThighLength,0>,LF_H2K);
        #local LF_Ankle=vrotate(<0,-CalfLength,0> ,LF_K2A);

        union {
 // Right Foot
                blob {  
                        threshold build-0.3
                        sphere {<0,-1,.5>,3,2}
                        sphere {0,3,2 scale <1.5,.5,.75> rotate y*-15 translate <0,-2,-8>}
                        cylinder {<0, 3,0><0,-2.5,-6>3,3 scale <1,.5,1>} 
                        cylinder {<0,1.5,0><0,-2.5,-7>3,2 rotate y*-10 scale <1,.5,1>} 
                        cylinder {<0,1.5,0><0,-2.5,-7>3,2 rotate y*  8 scale <1,.5,1>}
                        cylinder {<0,-2.5,0><0,-2.5,-6>3,2 scale <1,.65,1>} 
                        cylinder {<0,-2.5,0><0,-2.5,-7>3,2 rotate y*-10 scale <1,.65,1>} 
                        cylinder {<0,-2.5,0><0,-2.5,-7>3,2 rotate y*  8 scale <1,.65,1>}
                        sphere {0,1.5,2 scale <1,1,1.25> rotate x *-15 translate <-2.5,-2,-10>}
                        sphere {0,1.1,2 scale <1,1,1.25> rotate x *-15 translate <-1.0,-2,-10>}
                        sphere {0,1.0,2 scale <1,1,1.25> rotate x *-15 translate < 0.5,-2,-9.75>}
                        sphere {0,0.9,2 scale <1,1,1.25> rotate x *-15 translate < 2,-2,-9.25>}
                        sphere {0,0.8,2 scale <1,1,1.25> rotate x *-15 translate < 3.25,-2,-8.5>}
                        cylinder {<0,-5,0><0,5,0>5,-3 translate <-4,0,-5>}
                        cylinder {<0,-5,0><0,5,0>3,-2.5 scale <1,1,1.5> rotate z*-15 translate < 4,0,-4>}
        
                        scale <-.75,.75,.75>
                        translate y*-1
                        rotate RF_Rot
                        rotate RF_K2A
                        translate RF_Knee+RF_Ankle+<-4,-29,0>
                }       
// Left Foot
                blob {  
                        threshold build-0.3
                        sphere {<0,-1,.5>,3,2}
                        sphere {0,3,2 scale <1.5,.5,.75> rotate y*-15 translate <0,-2,-8>}
                        cylinder {<0, 3,0><0,-2.5,-6>3,3 scale <1,.5,1>} 
                        cylinder {<0,1.5,0><0,-2.5,-7>3,2 rotate y*-10 scale <1,.5,1>} 
                        cylinder {<0,1.5,0><0,-2.5,-7>3,2 rotate y*  8 scale <1,.5,1>}
                        cylinder {<0,-2.5,0><0,-2.5,-6>3,2 scale <1,.65,1>} 
                        cylinder {<0,-2.5,0><0,-2.5,-7>3,2 rotate y*-10 scale <1,.65,1>} 
                        cylinder {<0,-2.5,0><0,-2.5,-7>3,2 rotate y*  8 scale <1,.65,1>}
                        sphere {0,1.5,2 scale <1,1,1.25> rotate x *-15 translate <-2.5,-2,-10>}
                        sphere {0,1.1,2 scale <1,1,1.25> rotate x *-15 translate <-1.0,-2,-10>}
                        sphere {0,1.0,2 scale <1,1,1.25> rotate x *-15 translate < 0.5,-2,-9.75>}
                        sphere {0,0.9,2 scale <1,1,1.25> rotate x *-15 translate < 2,-2,-9.25>}
                        sphere {0,0.8,2 scale <1,1,1.25> rotate x *-15 translate < 3.25,-2,-8.5>}
                        cylinder {<0,-5,0><0,5,0>5,-3 translate <-4,0,-5>}
                        cylinder {<0,-5,0><0,5,0>3,-2.5 scale <1,1,1.5> rotate z*-15 translate < 4,0,-4>}

                        scale .75
                        translate y*-1
                        rotate LF_Rot
                        rotate LF_K2A
                        translate LF_Knee+LF_Ankle+<4,-29,0>
                }                 
                #if (DeBug=on)
                        union {
                                cylinder {<0,-3.25,0><0,-3.0,0>6.5 pigment {rgb <1,0,0>}}
                                object {LimbLoc1}
                                rotate LF_Rot
                                rotate LF_K2A
                                translate LF_Knee+LF_Ankle+<4,-29,0>                    
                        }
                        union {
                                cylinder {<0,-3.25,0><0,-3.0,0>6.5 pigment {rgb<1,0,0>}}
                                object {LimbLoc1}
                                rotate RF_Rot
                                rotate RF_K2A
                                translate RF_Knee+RF_Ankle+<-4,-29,0> 
                        }
                #end
                texture {Skin_Tex}
        }
#end

// Debuging Device

#ifndef (DeBug) 
        #declare DeBug=off; 
#end

#declare LimbLoc1=
        union {
                cylinder {<-8,0,0><8,0,0> .5 pigment {rgb <1,0,0>}}                
                cylinder {<0,0,-8><0,0,8> .5 pigment {rgb <0,1,0>}}
                cylinder {<0,-8,0><0,8,0> .5 pigment {rgb <0,0,1>}}
                torus {5,.4 pigment {rgb <1,1,0>}}
        }

#version Blob_Man_Version;

//EOF